@@ -9,7 +9,7 @@ class FFTermFactory:
     '''
     Factory class for force field terms.
 
-    Mainly used for loading a FFTerm from xml element stored in ZFP file.
+    Mainly used for loading a FFTerm from a line in ZFF file.
     '''
 
     _class_map = {}
@@ -17,7 +17,7 @@ class FFTermFactory:
     @staticmethod
     def register(klass, extra_names=None):
         '''
-        Register a force field term so that it can be read from or write into a ZFP or ZFF file.
+        Register a force field term so that it can be read from or write into a ZFF file.
 
         Parameters
         ----------
@@ -36,7 +36,7 @@ class FFTermFactory:
         Reconstruct a FFTerm using a dict of attributes read from ZFP xml file.
 
         Essentially it finds the class for the term, and calls the constructor of the class.
-        Every class to be reconstructed should have a class attribute `_zfp_attrs`.
+        Every class to be reconstructed should have a class attribute `_term_attrs`.
         This dict records which attributes should be saved into ZFP xml file,
         and it is also used as the arguments for initializing a FFTerm.
         It also tells the data type of these attributes.
@@ -65,15 +65,15 @@ class FFTermFactory:
             raise Exception('Invalid force field term: %s' % name)
         kwargs = {}
         adjustables = []
-        for attr, func in cls._zfp_attrs.items():
+        for attr, func in cls._term_attrs.items():
             str_val = d[attr]
             if str_val.endswith('?'):
                 str_val = str_val[:-1]
                 if func is float:
                     adjustables.append(attr)
             val = func(str_val)
-            if attr in cls._zfp_convert:
-                val = cls._zfp_convert[attr][0](val)
+            if attr in cls._attr_convert:
+                val = cls._attr_convert[attr][0](val)
             kwargs[attr] = val
         try:
             term = cls(**kwargs)
@@ -89,10 +89,10 @@ class FFTermFactory:
         Reconstruct a FFTerm using a line read from ZFF text file.
 
         Essentially it finds the class for the term, and calls the constructor of the class.
-        Every class to be reconstructed should have a class attribute `_zfp_attrs`.
-        This dict records which attributes should be saved into ZFP xml file,
+        Every class to be reconstructed should have a class attribute `_term_attrs`.
+        This dict records which attributes should be saved into ZFF file,
         and it is also used as the arguments for initializing a FFTerm.
-        It also tell the data type of these attributes.
+        It also tells the data type of these attributes.
 
         Some force field terms can not be fully built from the constructor.
         Then :func:`FFTerm._from_zff_extra` should be implemented for those terms,
@@ -118,14 +118,14 @@ class FFTermFactory:
             raise Exception('Invalid force field term: %s' % name)
         kwargs = {}
         adjustables = []
-        for (attr, func), str_val in zip(cls._zfp_attrs.items(), words[1:]):
+        for (attr, func), str_val in zip(cls._term_attrs.items(), words[1:]):
             if str_val.endswith('?'):
                 str_val = str_val[:-1]
                 if func is float:
                     adjustables.append(attr)
             val = func(str_val)
-            if attr in cls._zfp_convert:
-                val = cls._zfp_convert[attr][0](val)
+            if attr in cls._attr_convert:
+                val = cls._attr_convert[attr][0](val)
             kwargs[attr] = val
         try:
             term = cls(**kwargs)
@@ -149,22 +149,22 @@ class FFTerm:
         Comments is useful for debugging when generating input files for simulation engines.
     '''
 
-    _zfp_attrs = {}
+    _term_attrs = {}
     '''
-    The attributes that will be saved into or loaded from ZFP or ZFF file.
+    The attributes that will be saved into or loaded from ZFF file.
     The key is the attribute name. The value is a function to parse the string to object.
-    This dict must be overridden by every subclasses that can be saved into or reconstructed from ZFP or ZFF file.
+    This dict must be overridden by every subclasses that can be saved into or reconstructed from ZFF file.
     '''
 
-    _zfp_convert = {}
+    _attr_convert = {}
     '''
-    The parameters that will be converted when read/write parameters from/to ZFP or ZFF file.
+    The parameters that will be converted when read/write parameters from/to ZFF file.
     The key is the attribute name. The value is a tuple of two functions.
     The first one convert data from external to internal. The second one do the opposite.
-    For example, the angle is in unit of radian internally. But it is in unit of degree in ZFP or ZFF file.
-    Therefore, for AngleTerm, _zfp_convert have two functions for attribute 'theta'.
+    For example, the angle is in unit of radian internally. But it is in unit of degree in ZFF file.
+    Therefore, for AngleTerm, _attr_convert have two functions for attribute 'theta'.
     The first one convert degree to radian, and the second one convert radian to degree.
-    This dict must be overridden by subclasses that have different value internally and in ZFP or ZFF file.
+    This dict must be overridden by subclasses that have different value internally and in ZFF file.
     '''
 
     @classmethod
@@ -195,42 +195,6 @@ class FFTerm:
         '''
         raise NotImplementedError('This property should be implemented by subclasses')
 
-    def to_zfp(self):
-        '''
-        Pack the attributes of a term into a dict so that can be saved into ZFP xml file.
-
-        Every class to be packed should have a class property `zfp_attrs`.
-        This dict records which attributes should be saved into ZFP xml file,
-        and it is also used as the arguments for initializing a FFTerm.
-        It also tell the data type of these attributes.
-
-        Returns
-        -------
-        d : dict, [str, str]
-        '''
-        d = {}
-        for attr, func in self._zfp_attrs.items():
-            val = getattr(self, attr)
-            if attr in self._zfp_convert:
-                val = self._zfp_convert[attr][1](val)
-            d[attr] = str(val)
-            if func is float and attr in self.adjustables:
-                d[attr] += '?'
-        d.update(self._to_zfp_extra())
-        return d
-
-    def _to_zfp_extra(self):
-        '''
-        Pack extra information for some special terms.
-
-        e.g. PeriodicDihedralTerm can contain arbitrary numbers of dihedral parameters.
-
-        Returns
-        -------
-        d : dict, [str, str]
-        '''
-        return {}
-
     def _from_zfp_extra(self, d):
         '''
         Some force field terms can not be fully built from the constructor.
@@ -248,8 +212,8 @@ class FFTerm:
         '''
         Pack the attributes of a term into a string so that can be saved into a line in ZFF file.
 
-        Every class to be packed should have a class property `zfp_attrs`.
-        This dict records which attributes should be saved into ZFP xml file,
+        Every class to be packed should have a class property `_term_attrs`.
+        This dict records which attributes should be saved into ZFF file,
         and it is also used as the arguments for initializing a FFTerm.
         It also tells the data type of these attributes.
 
@@ -258,22 +222,24 @@ class FFTerm:
         line : string
         '''
         line = '%-16s' % self.__class__.get_alias()
-        for attr, func in self._zfp_attrs.items():
+        for attr, func in self._term_attrs.items():
             val = getattr(self, attr)
-            if attr in self._zfp_convert:
-                val = self._zfp_convert[attr][1](val)
-            if type(val) is float:
-                if val > 1000:
+            if attr in self._attr_convert:
+                val = self._attr_convert[attr][1](val)
+            if func is float:
+                if abs(val) >= 1E6:
+                    string = ' %8i' % round(val)
+                elif abs(val) >= 1E3:
                     string = ' %8.1f' % val
+                elif 0 < abs(val) < 1E-3:
+                    string = ' %8.1e' % val
                 else:
                     string = ' %8.4f' % val
-            elif type(val) is int:
+                string += '?' if attr in self.adjustables else ' '
+            elif func is int:
                 string = ' %2i' % val
-            elif type(val) is bool:
-                string = ' %6s' % val
             else:
                 string = ' %-9s' % val
-            string += '?' if func is float and attr in self.adjustables else ' '
             line += string
 
         return line.rstrip()
@@ -287,16 +253,11 @@ class FFTerm:
         header : str
         '''
         line = '#%-15s' % self.__class__.get_alias()
-        for attr in self._zfp_attrs:
-            val = getattr(self, attr)
-            if attr in self._zfp_convert:
-                val = self._zfp_convert[attr][1](val)
-            if type(val) is float:
+        for attr, func in self._term_attrs.items():
+            if func is float:
                 string = ' %8s ' % attr
-            elif type(val) is int:
-                string = ' %2s' % attr
-            elif type(val) is bool:
-                string = ' %6s' % attr
+            elif func is int:
+                string = ' %2i' % attr
             else:
                 string = ' %-9s' % attr
             line += string
@@ -351,7 +312,7 @@ class AtomType(FFTerm):
         The mass of this atom type.
     charge : float , optional
         The charge of this atom type.
-    eqt_q_inc : str , optional
+    eqt_bci : str , optional
         The equivalent type for bond charge increment parameters
     eqt_vdw : str , optional
         The equivalent type for vdW parameters
@@ -382,7 +343,7 @@ class AtomType(FFTerm):
         The charge of this atom type.
         Similar to mass, the charges we are finally going to use are from the :class:`~mstk.topology.Topology`.
         But charges in force field can be used to assign the charges in topology.
-    eqt_q_inc : str
+    eqt_bci : str
         The equivalent type for bond charge increment parameters
     eqt_vdw : str
         The equivalent type for vdW parameters
@@ -403,11 +364,11 @@ class AtomType(FFTerm):
     eqt_polar : str
         The equivalent type for polarization parameters
     '''
-    _zfp_attrs = {
+    _term_attrs = {
         'name'     : str,
         'mass'     : float,
         'charge'   : float,
-        'eqt_q_inc': str,
+        'eqt_bci'  : str,
         'eqt_vdw'  : str,
         'eqt_bond' : str,
         'eqt_ang_c': str,
@@ -420,16 +381,18 @@ class AtomType(FFTerm):
     }
 
     def __init__(self, name, mass=-1, charge=0,
-                 eqt_q_inc=None, eqt_vdw=None, eqt_bond=None,
+                 eqt_bci=None, eqt_vdw=None, eqt_bond=None,
                  eqt_ang_c=None, eqt_ang_s=None, eqt_dih_c=None, eqt_dih_s=None,
                  eqt_imp_c=None, eqt_imp_s=None, eqt_polar=None):
         super().__init__()
+        # cannot set name directly because name is a property for each FFTerm subclass
+        # to allow create_from_zff() work correctly for AtomType, the name must be set with @name.setter
         self._name = name
         # mass is only used for assign mass of topology in case mass not exist in topology
-        # mass = -1 means unknown
+        # -1 means mass unknown
         self.mass = mass
         self.charge = charge
-        self.eqt_q_inc = eqt_q_inc or name
+        self.eqt_bci = eqt_bci or name
         self.eqt_vdw = eqt_vdw or name
         self.eqt_bond = eqt_bond or name
         self.eqt_ang_c = eqt_ang_c or name
@@ -455,12 +418,11 @@ class AtomType(FFTerm):
 
     @name.setter
     def name(self, value):
-        # this is a trick allowing from_zfp() to work correctly
         self._name = value
 
     @property
     def eqt_types(self):
-        return [self.eqt_q_inc,
+        return [self.eqt_bci,
                 self.eqt_vdw,
                 self.eqt_bond,
                 self.eqt_ang_c,
@@ -515,7 +477,7 @@ class ChargeIncrementTerm(FFTerm):
     value : float
         The offset from type2 to type1
     '''
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'value': float,
@@ -652,7 +614,7 @@ class AngleTerm(FFTerm):
     '''
 
     # the angle in ZFP or ZFF file is in unit of degree. one digit is enough
-    _zfp_convert = {
+    _attr_convert = {
         'theta': (lambda x: x * DEG2RAD, lambda x: round(x * RAD2DEG, 2))
     }
 
@@ -817,12 +779,12 @@ class ImproperTerm(FFTerm):
                > [other.type1, other.type2, other.type3, other.type4]
 
 
-class PolarizableTerm(FFTerm):
+class PolarTerm(FFTerm):
     '''
     Base class for all polarization terms.
 
     Subclasses should be implemented for each functional form,
-    e.g. :class:`DrudeTerm`.
+    e.g. :class:`DrudePolarTerm`.
 
     Parameters
     ----------
@@ -876,7 +838,7 @@ class LJ126Term(VdwTerm):
 
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1'  : str,
         'type2'  : str,
         'epsilon': float,
@@ -928,7 +890,7 @@ class MieTerm(VdwTerm):
     attraction : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1'     : str,
         'type2'     : str,
         'epsilon'   : float,
@@ -993,6 +955,57 @@ class MieTerm(VdwTerm):
 FFTermFactory.register(MieTerm)
 
 
+class MorseVdwTerm(VdwTerm):
+    '''
+    vdW term with Morse function form.
+
+    The energy function is
+
+    >>> U = depth*((1-exp(-alpha*(r-r0)))^2-1)
+
+    Note its difference to the MorseBondTerm. MorseVdwTerm has zero energy when two atoms are far away,
+    while MorseBondTerm has zero energy at equilibrium distance.
+
+    During the initialization, the two atom types will be sorted by their string.
+
+    Parameters
+    ----------
+    type1 : str
+    type2 : str
+    depth: float
+    alpha: float
+    r0: float
+
+    Attributes
+    ----------
+    type1 : str
+    type2 : str
+    depth: float
+    alpha: float
+    r0: float
+    '''
+
+    _term_attrs = {
+        'type1': str,
+        'type2': str,
+        'depth': float,
+        'alpha': float,
+        'r0'   : float,
+    }
+
+    def __init__(self, type1, type2, depth, alpha, r0):
+        super().__init__(type1, type2)
+        self.depth = depth
+        self.alpha = alpha
+        self.r0 = r0
+
+    def evaluate_energy(self, r):
+        return self.depth * ((1 - math.exp(-self.alpha * (r - self.r0))) ** 2 - 1)
+
+
+FFTermFactory.register(MorseVdwTerm)
+
+
 class HarmonicBondTerm(BondTerm):
     '''
     Bond term in harmonic function form
@@ -1020,7 +1033,7 @@ class HarmonicBondTerm(BondTerm):
     fixed : bool
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1' : str,
         'type2' : str,
         'length': float,
@@ -1039,6 +1052,61 @@ class HarmonicBondTerm(BondTerm):
 FFTermFactory.register(HarmonicBondTerm)
 
 
+class QuarticBondTerm(BondTerm):
+    '''
+    Bond term in quartic function form
+
+    The energy function is
+
+    >>> U = k2 * (b-b0)^2 + k3 * (b-b0)^3 + k4 * (b-b0)^4
+
+    During the initialization, the two atom types will be sorted by their string.
+
+    Parameters
+    ----------
+    type1 : str
+    type2 : str
+    length : float
+    k2 : float
+    k3 : float
+    k4 : float
+    fixed : bool
+
+    Attributes
+    ----------
+    type1 : str
+    type2 : str
+    length : float
+    k2 : float
+    k3 : float
+    k4 : float
+    fixed : bool
+    '''
+
+    _term_attrs = {
+        'type1' : str,
+        'type2' : str,
+        'length': float,
+        'k2'    : float,
+        'k3'    : float,
+        'k4'    : float,
+        'fixed' : lambda x: bool(strtobool(x))
+    }
+
+    def __init__(self, type1, type2, length, k2, k3, k4, fixed=False):
+        super().__init__(type1, type2, length, fixed=fixed)
+        self.k2 = k2
+        self.k3 = k3
+        self.k4 = k4
+
+    def evaluate_energy(self, val):
+        return self.k2 * (val - self.length) ** 2 + self.k3 * (val - self.length) ** 3 \
+               + self.k4 * (val - self.length) ** 4
+
+
+FFTermFactory.register(QuarticBondTerm)
+
+
 class MorseBondTerm(BondTerm):
     '''
     Bond term in Morse function form
@@ -1050,6 +1118,9 @@ class MorseBondTerm(BondTerm):
 
     The `k` corresponds to the force constant `k` in HarmonicBondTerm.
 
+    Note its difference to the MorseVdwTerm. MorseVdwTerm has zero energy when two atoms are far away,
+    while MorseBondTerm has zero energy at equilibrium distance.
+
     During the initialization, the two atom types will be sorted by their string.
 
     Parameters
@@ -1069,7 +1140,7 @@ class MorseBondTerm(BondTerm):
     depth : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1' : str,
         'type2' : str,
         'length': float,
@@ -1121,7 +1192,7 @@ class HarmonicAngleTerm(AngleTerm):
     fixed : bool
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1137,14 +1208,121 @@ class HarmonicAngleTerm(AngleTerm):
     def evaluate_energy(self, val):
         return self.k * (val - self.theta) ** 2
 
-    @property
-    def is_linear(self):
-        return self.theta > 175 * DEG2RAD
-
 
 FFTermFactory.register(HarmonicAngleTerm)
 
 
+class QuarticAngleTerm(AngleTerm):
+    '''
+    Angle term in quartic function form
+
+    The energy function is
+
+    >>> U = k2 * (theta-theta0)^2 + k3 * (theta-theta0)^3 + k4 * (theta-theta0)^4
+
+    The angle is in unit of radian, and the force constant is in unit of kJ/mol/rad^2.
+
+    During the initialization, the two side atom types will be sorted by their string.
+
+    Parameters
+    ----------
+    type1 : str
+    type2 : str
+    type3 : str
+    theta : float
+    k2 : float
+    k3 : float
+    k4 : float
+    fixed : bool
+
+    Attributes
+    ----------
+    type1 : str
+    type2 : str
+    type3 : str
+    theta : float
+    k2 : float
+    k3 : float
+    k4 : float
+    fixed : bool
+    '''
+
+    _term_attrs = {
+        'type1': str,
+        'type2': str,
+        'type3': str,
+        'theta': float,
+        'k2'   : float,
+        'k3'   : float,
+        'k4'   : float,
+        'fixed': lambda x: bool(strtobool(x))
+    }
+
+    def __init__(self, type1, type2, type3, theta, k2, k3, k4, fixed=False):
+        super().__init__(type1, type2, type3, theta, fixed=fixed)
+        self.k2 = k2
+        self.k3 = k3
+        self.k4 = k4
+
+    def evaluate_energy(self, val):
+        return self.k2 * (val - self.theta) ** 2 + self.k3 * (val - self.theta) ** 3 \
+               + self.k4 * (val - self.theta) ** 4
+
+
+FFTermFactory.register(QuarticAngleTerm)
+
+
+class HarmonicCosineAngleTerm(AngleTerm):
+    '''
+    Angle term in harmonic cosine function form
+
+    The energy function is
+
+    >>> U = k / sin(theta0)^2 * (cos(theta) - cos(theta0))^2
+
+    The angle is in unit of radian, and the force constant is in unit of kJ/mol/rad^2.
+
+    During the initialization, the two side atom types will be sorted by their string.
+
+    Parameters
+    ----------
+    type1 : str
+    type2 : str
+    type3 : str
+    theta : float
+    k : float
+    fixed : bool
+
+    Attributes
+    ----------
+    type1 : str
+    type2 : str
+    type3 : str
+    theta : float
+    k : float
+    fixed : bool
+    '''
+
+    _term_attrs = {
+        'type1': str,
+        'type2': str,
+        'type3': str,
+        'theta': float,
+        'k'    : float,
+        'fixed': lambda x: bool(strtobool(x))
+    }
+
+    def __init__(self, type1, type2, type3, theta, k, fixed=False):
+        super().__init__(type1, type2, type3, theta, fixed=fixed)
+        self.k = k
+
+    def evaluate_energy(self, val):
+        return self.k / math.sin(self.theta) ** 2 * (math.cos(val) - math.cos(self.theta)) ** 2
+
+
+FFTermFactory.register(HarmonicCosineAngleTerm)
+
+
 class SDKAngleTerm(AngleTerm):
     '''
     Angle term in SDK function form
@@ -1181,7 +1359,7 @@ class SDKAngleTerm(AngleTerm):
     fixed : bool
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1193,10 +1371,6 @@ class SDKAngleTerm(AngleTerm):
         super().__init__(type1, type2, type3, theta, fixed=False)
         self.k = k
 
-    @property
-    def is_linear(self):
-        return self.theta > 175 * DEG2RAD
-
 
 FFTermFactory.register(SDKAngleTerm)
 
@@ -1240,7 +1414,7 @@ class LinearAngleTerm(AngleTerm):
     fixed : bool
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1256,10 +1430,6 @@ class LinearAngleTerm(AngleTerm):
         k_linear = self.k * 2 * (b12 + b23) ** 2 / (b12 * b23) ** 2
         return a, k_linear
 
-    @property
-    def is_linear(self):
-        return True
-
 
 FFTermFactory.register(LinearAngleTerm)
 
@@ -1270,7 +1440,7 @@ class OplsDihedralTerm(DihedralTerm):
 
     The energy function is
 
-    >>> U = k_1*(1+cos(phi)) + k_2*(1-cos(2*phi)) + k_3*(1+cos(3*phi)) - k_4*(1-cos(4*phi))
+    >>> U = k_1*(1+cos(phi)) + k_2*(1-cos(2*phi)) + k_3*(1+cos(3*phi)) + k_4*(1-cos(4*phi))
 
     OPLS form is commonly used, therefore we don't generalize it with :class:`PeriodicDihedralTerm`.
 
@@ -1305,7 +1475,7 @@ class OplsDihedralTerm(DihedralTerm):
     k4 : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1411,7 +1581,7 @@ class PeriodicDihedralTerm(DihedralTerm):
         Each element is a namedtuple :attr:`Phase` describing the parameters for each phase.
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1454,25 +1624,14 @@ class PeriodicDihedralTerm(DihedralTerm):
         self.phases.append(PeriodicDihedralTerm.Phase(phi=phi, k=k, n=n))
         self.phases.sort(key=lambda x: x.n)
 
-    def _to_zfp_extra(self) -> {str: str}:
-        d = {}
-        for phase in self.phases:
-            d.update({
-                'phi_%i' % phase.n: '%.1f' % (phase.phi * RAD2DEG),  # the angle in ZFP or ZFF file is in unit of degree
-                'k_%i' % phase.n  : '%.4f' % phase.k,
-            })
-        return d
-
     def _from_zfp_extra(self, d: {str: str}):
         for key in d.keys():
             if key.startswith('phi_'):
                 n = int(key.split('_')[-1])
-                phi = float(d['phi_%i' % n]) * DEG2RAD  # the angle in ZFP or ZFF file is in unit of degree
+                phi = float(d['phi_%i' % n]) * DEG2RAD  # the angle in ZFP file is in unit of degree
                 k = float(d['k_%i' % n])
                 if k != 0:
                     self.add_phase(phi, k, n)
-        if not self.phases:
-            self.add_phase(0.0, 0.0, 1)
 
     def to_zff(self):
         line = '%-16s %-9s %-9s %-9s %-9s' % (self.__class__.get_alias(),
@@ -1488,13 +1647,11 @@ class PeriodicDihedralTerm(DihedralTerm):
         if len(str_vals) % 3 != 0:
             raise Exception(f'Invalid parameters for {self.__class__.__name__}')
         for i in range(len(str_vals) // 3):
-            phi = float(str_vals[i * 3]) * DEG2RAD  # the angle in ZFP or ZFF file is in unit of degree
+            phi = float(str_vals[i * 3]) * DEG2RAD  # the angle in ZFF file is in unit of degree
             k = float(str_vals[i * 3 + 1])
             n = int(str_vals[i * 3 + 2])
             if k != 0:
                 self.add_phase(phi, k, n)
-        if not self.phases:
-            self.add_phase(0.0, 0.0, 1)
 
     def evaluate_energy(self, val):
         energy = 0
@@ -1521,26 +1678,36 @@ class PeriodicDihedralTerm(DihedralTerm):
         -------
         term : OplsDihedralTerm
         '''
-        k1 = k2 = k3 = k4 = 0.0
-        for para in self.phases:
-            if para.n == 1:
-                if para.phi != 0:
-                    raise Exception(f'{str(self)} does not follow OPLS convention, phi_1 != 0')
-                k1 = para.k
-            elif para.n == 2:
-                if abs(para.phi - PI) > 0.01:
-                    raise Exception(f'{str(self)} does not follow OPLS convention, phi_2 != PI')
-                k2 = para.k
-            elif para.n == 3:
-                if para.phi != 0:
-                    raise Exception(f'{str(self)} does not follow OPLS convention, phi_3 != 0')
-                k3 = para.k
-            elif para.n == 4:
-                if abs(para.phi - PI) > 0.01:
-                    raise Exception(f'{str(self)} does not follow OPLS convention, phi_4 != PI')
-                k4 = para.k
-            else:
+        phases = [p for p in self.phases if p.k != 0]
+        for phase in phases:
+            if phase.n > 4:
                 raise Exception(f'{str(self)} does not follow OPLS convention, n > 4')
+        for phase in phases:
+            if phase.phi != 0 and abs(phase.phi - PI) > 0.01:
+                raise Exception(f'{str(self)} does not follow OPLS convention, phi != 0 or PI')
+
+        k1 = k2 = k3 = k4 = 0.0
+        for phase in phases:
+            if phase.n == 1:
+                if phase.phi == 0:
+                    k1 += phase.k
+                else:
+                    k1 -= phase.k
+            elif phase.n == 2:
+                if abs(phase.phi - PI) < 0.01:
+                    k2 += phase.k
+                else:
+                    k2 -= phase.k
+            elif phase.n == 3:
+                if phase.phi == 0:
+                    k3 += phase.k
+                else:
+                    k3 -= phase.k
+            elif phase.n == 4:
+                if abs(phase.phi - PI) < 0.01:
+                    k4 += phase.k
+                else:
+                    k4 -= phase.k
 
         return OplsDihedralTerm(self.type1, self.type2, self.type3, self.type4, k1, k2, k3, k4)
 
@@ -1583,7 +1750,7 @@ class OplsImproperTerm(ImproperTerm):
     k : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1641,7 +1808,7 @@ class HarmonicImproperTerm(ImproperTerm):
     k : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type1': str,
         'type2': str,
         'type3': str,
@@ -1651,7 +1818,7 @@ class HarmonicImproperTerm(ImproperTerm):
     }
 
     # the angle in ZFP or ZFF file is in unit of degree. one digit is enough
-    _zfp_convert = {
+    _attr_convert = {
         'phi': (lambda x: x * DEG2RAD, lambda x: round(x * RAD2DEG, 2))
     }
 
@@ -1667,7 +1834,7 @@ class HarmonicImproperTerm(ImproperTerm):
 FFTermFactory.register(HarmonicImproperTerm)
 
 
-class DrudeTerm(PolarizableTerm):
+class DrudePolarTerm(PolarTerm):
     '''
     Polarization term described by Drude induced dipole.
 
@@ -1682,7 +1849,7 @@ class DrudeTerm(PolarizableTerm):
 
     If this atom is bonded with hydrogen atoms, the argument `merge_alpha_H` determines the polarizability of neighbour H atoms.
     The alpha of these H atoms will be merged into the this atom.
-    Therefore, DrudeTerm for H atom types should not be provided explicitly. Otherwise, there will be double count.
+    Therefore, DrudePolarTerm for H atom types should not be provided explicitly. Otherwise, there will be double count.
     The H atoms are identified by check the :attr:`~mstk.topology.Atom.symbol` attribute.
     An atom is considered to be hydrogen if symbol equals 'H'.
 
@@ -1705,7 +1872,7 @@ class DrudeTerm(PolarizableTerm):
     merge_alpha_H : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type'         : str,
         'alpha'        : float,
         'thole'        : float,
@@ -1722,6 +1889,12 @@ class DrudeTerm(PolarizableTerm):
         self.mass = mass
         self.merge_alpha_H = merge_alpha_H
 
+    def to_zff(self):
+        line = '%-16s %-9s %8.3e  %8.4f  %8.1f  %8.4f  %8.3e' % (self.__class__.get_alias(),
+                                                                 self.type, self.alpha, self.thole, self.k, self.mass,
+                                                                 self.merge_alpha_H)
+        return line
+
     def get_charge(self, alpha=None):
         '''
         Get the charge offset between parent atom and Drude particle.
@@ -1744,7 +1917,7 @@ class DrudeTerm(PolarizableTerm):
         return math.sqrt(4 * PI * VACUUM_PERMITTIVITY * (2 * self.k) * alpha) * factor
 
 
-FFTermFactory.register(DrudeTerm)
+FFTermFactory.register(DrudePolarTerm)
 
 
 class VirtualSiteTerm(FFTerm):
@@ -1752,7 +1925,7 @@ class VirtualSiteTerm(FFTerm):
     Base class for all virtual site terms.
 
     Subclasses should be implemented for each functional form,
-    e.g. :class:`TIP4PTerm`.
+    e.g. :class:`TIP4PSiteTerm`.
 
     Parameters
     ----------
@@ -1797,7 +1970,7 @@ class TIP4PSiteTerm(VirtualSiteTerm):
     d : float
     '''
 
-    _zfp_attrs = {
+    _term_attrs = {
         'type'  : str,
         'type_O': str,
         'type_H': str,
