@@ -5,7 +5,7 @@ from mstk.forcefield.errors import *
 from mstk import logger
 
 
-class Padua():
+class Padua:
     '''
     Generate ForceField from the force field file of `fftool`.
 
@@ -17,28 +17,26 @@ class Padua():
     * :class:`~mstk.forcefield.HarmonicAngleTerm`
     * :class:`~mstk.forcefield.PeriodicDihedralTerm`
     * :class:`~mstk.forcefield.OplsImproperTerm`
-    * :class:`~mstk.forcefield.DrudeTerm`
+    * :class:`~mstk.forcefield.DrudePolarTerm`
 
     The energy terms are in k/2 form for bond, angle, dihedral and improper.
     The length is in unit of Angstrom, and angle is in unit of degree.
     The energies for bond, angle and vdW are in unit of kJ/mol/A^2, kJ/mol/rad^2 and kJ/mol.
-
-    Several files can be read at one time.
-    Only one ForceField object will be created and it will contain force field terms from all of the files.
+    The two values for LJ interaction are sigma and epsilon.
 
     A force field term describing one topology element should only appear once.
     If there are duplicated terms, an Exception will be raised.
 
     Parameters
     ----------
-    files : list of str
+    file : str
 
     Attributes
     ----------
     forcefield : ForceField
     '''
 
-    def __init__(self, *files):
+    def __init__(self, file):
         ff = ForceField()
         ff.lj_mixing_rule = ForceField.LJ_MIXING_GEOMETRIC
         ff.vdw_long_range = ForceField.VDW_LONGRANGE_CORRECT
@@ -46,8 +44,7 @@ class Padua():
         ff.scale_14_vdw = 0.5
         ff.scale_14_coulomb = 0.5
 
-        for file in files:
-            self._parse(ff, file)
+        self._parse(ff, file)
         self._setup(ff)
 
         self.forcefield = ff
@@ -106,12 +103,12 @@ class Padua():
                 ff.add_term(dtype)
                 ff.add_term(vdw)
 
-        # If H* is defined in polarizable term, alpha of H will be merged into attached heavy atoms and the term will be removed
-        if 'H*' in ff.polarizable_terms:
-            hterm = ff.polarizable_terms.pop('H*')
-            for pterm in ff.polarizable_terms.values():
+        # If H* is defined in polar term, alpha of H will be merged into attached heavy atoms and the term will be removed
+        if 'H*' in ff.polar_terms:
+            hterm = ff.polar_terms.pop('H*')
+            for pterm in ff.polar_terms.values():
                 pterm.merge_alpha_H = hterm.alpha
-            logger.info(f'H* found in polarizable term. '
+            logger.info(f'H* found in polar term. '
                         f'Polarizability of H will be merged into attached heavy atoms')
 
     def _parse_atom(self, ff, words):
@@ -155,12 +152,8 @@ class Padua():
     def _parse_dihedral(self, ff, words):
         if words[4] not in ['opls']:
             raise Exception('Unsupported dihedral function: %s' % (words[4]))
-        term = PeriodicDihedralTerm(words[0], words[1], words[2], words[3])
         k1, k2, k3, k4 = list(map(lambda x: float(x) / 2, words[5:9]))
-        if k1 != 0: term.add_phase(0.0, k1, 1)
-        if k2 != 0: term.add_phase(PI, k2, 2)
-        if k3 != 0: term.add_phase(0.0, k3, 3)
-        if k4 != 0: term.add_phase(PI, k4, 4)
+        term = OplsDihedralTerm(words[0], words[1], words[2], words[3], k1, k2, k3, k4)
         if term.name in ff.dihedral_terms.keys():
             raise Exception('Duplicated dihedral term: %s' % str(term))
         ff.dihedral_terms[term.name] = term
@@ -182,15 +175,15 @@ class Padua():
 
     def _parse_polarization(slef, ff, words):
         name, mass, charge, k, alpha, thole = words
-        term = DrudeTerm(name, float(alpha) / 1000, float(thole))
+        term = DrudePolarTerm(name, float(alpha) / 1000, float(thole))
         term.mass = float(mass)
         term.k = float(k) / 2 * 100  # convert from kJ/mol/A^2 to kJ/mol/nm^2
-        if term.name in ff.polarizable_terms.keys():
+        if term.name in ff.polar_terms.keys():
             raise Exception('Duplicated drude term: %s' % str(term))
-        ff.polarizable_terms[term.name] = term
+        ff.polar_terms[term.name] = term
 
 
-class Monomer():
+class Monomer:
     def __init__(self, name: str, charge: float, dipole: float, alpha: float = None):
         self.name = name
         self.charge = charge
@@ -202,7 +195,7 @@ class Monomer():
         return '<Monomer: %s>' % self.name
 
 
-class Dimer():
+class Dimer:
     C0 = 0.25
     C1 = 0.11
 
@@ -233,9 +226,9 @@ class Dimer():
         self.scale_factor = round(1 / (1 + k), 3)
 
 
-class PaduaLJScaler():
+class PaduaLJScaler:
     '''
-    PaduaLJScaler implements the empirical LJ scaling scheme of Goloviznina Kateryna.
+    PaduaLJScaler implements the empirical LJ scaling scheme of Goloviznina Kateryna.
 
     A scaling file should be providing,
     which provides information about fragments, dimers, atom types belongs to each fragments,
@@ -430,4 +423,4 @@ class PaduaLJScaler():
         return dimer.scale_factor
 
 
-ForceField.registor_format('.ff', Padua)
+ForceField.register_format('.ff', Padua)
