@@ -27,11 +27,14 @@ def print_omm_info(logger=None):
 
 def minimize(sim, tolerance, gro_out=None, logger=None):
     '''
-    Run energy minimization on a Simulation until the force on each atom is smaller than tolerance.
+    Run energy minimization on a Simulation until the force on every atom is smaller than tolerance.
 
     Note that the tolerance here differs from that in OpenMM LocalEnergyMinimizer.
     In OpenMM, the tolerance is set for the root-mean-square of N*3 force components in the whole system.
 
+    For system with constraints, the max force may never reach the tolerance.
+    This is because OpenMM uses harmonic restraints instead of rigorous constrints during energy minimization.
+
     Parameters
     ----------
     sim : app.Simulation
@@ -52,9 +55,9 @@ def minimize(sim, tolerance, gro_out=None, logger=None):
         fmax = fsq[imax] ** 0.5
         if logger:
             logger.info(f'Iter {i1_iter} Max force F_{imax} = {fmax} kJ/mol/nm')
-        if fmax < tolerance or i1_iter >= 10:
+        if fmax < tolerance or i1_iter >= 5:
             break
-        tol_iter /= 2
+        tol_iter /= 4
 
     state = sim.context.getState(getPositions=True, getEnergy=True)
     if logger:
@@ -64,17 +67,21 @@ def minimize(sim, tolerance, gro_out=None, logger=None):
         GroFile.writeFile(sim.topology, state.getPositions(), state.getPeriodicBoxVectors(), gro_out)
 
 
-def apply_mc_barostat(system, pcoupl, P, T, nstep=100, rigid_molecule=True, logger=None):
+def apply_mc_barostat(system, type, P, T, nstep=100, rigid_molecule=True, logger=None):
     '''
     Add a MonteCarlo barostat to the system and return the added barostat
 
-    TODO It requires the forked OpenMM https://github.com/openmm/openmm/pull/3797
+    Notes
+    -----
+    It requires the OpenMM with barostat patched https://github.com/openmm/openmm/pull/3797
 
     Parameters
     ----------
     system : mm.System
-    pcoupl : str
-        The type of barostat. Can be one of the following: iso, semi-iso, aniso, xy, z.
+    type : str
+        The type of barostat.
+        Can be one of the following: iso, aniso, xyz-coupled, xy-coupled-z, xyz, xy-coupled, xy, z.
+        iso is alias of xyz-coupled; aniso is alias of xyz
     P : float
         Pressure in bar
     T : float
@@ -88,26 +95,33 @@ def apply_mc_barostat(system, pcoupl, P, T, nstep=100, rigid_molecule=True, logg
     barostat : mm.Force
         The added barostat force
     '''
-    if pcoupl == 'iso':
+    if type in ('xyz-coupled', 'iso'):
         msg = 'Isotropic barostat'
         force = mm.MonteCarloBarostat(P * bar, T * kelvin, nstep, rigid_molecule)
-    elif pcoupl == 'semi-iso':
-        msg = 'Anisotropic barostat with coupled XY'
+    elif type == 'xy-coupled-z':
+        msg = 'Anisotropic barostat with XY coupled'
         force = mm.MonteCarloMembraneBarostat(P * bar, 0 * bar * nm, T * kelvin,
                                               mm.MonteCarloMembraneBarostat.XYIsotropic,
                                               mm.MonteCarloMembraneBarostat.ZFree,
                                               nstep, rigid_molecule)
-    elif pcoupl == 'aniso':
+    elif type in ('xyz', 'aniso'):
         msg = 'Anisotropic barostat'
         force = mm.MonteCarloAnisotropicBarostat([P * bar] * 3, T * kelvin, True, True, True, nstep, rigid_molecule)
-    elif pcoupl == 'xy':
+    elif type == 'xy-coupled':
+        msg = 'Anisotropic barostat with XY coupled and Z fixed'
+        force = mm.MonteCarloMembraneBarostat(P * bar, 0 * bar * nm, T * kelvin,
+                                              mm.MonteCarloMembraneBarostat.XYIsotropic,
+                                              mm.MonteCarloMembraneBarostat.ZFixed,
+                                              nstep, rigid_molecule)
+    elif type == 'xy':
         msg = 'Anisotropic barostat only for X and Y'
         force = mm.MonteCarloAnisotropicBarostat([P * bar] * 3, T * kelvin, True, True, False, nstep, rigid_molecule)
-    elif pcoupl == 'z':
+    elif type == 'z':
         msg = 'Anisotropic barostat only for Z'
         force = mm.MonteCarloAnisotropicBarostat([P * bar] * 3, T * kelvin, False, False, True, nstep, rigid_molecule)
     else:
-        raise Exception('Available pressure coupling types: iso, semi-iso, aniso, xy, z')
+        raise Exception(
+            'Available pressure coupling types: iso, aniso, xyz-coupled, xy-coupled-z, xyz, xy-coupled, xy, z')
 
     scaled_unit = 'molecules' if rigid_molecule else 'constrained groups'
     msg += f' with {scaled_unit} scaled'
