@@ -1,4 +1,4 @@
 from .pbsjob import PbsJob
-from .scheduler import Scheduler
+from .scheduler import Scheduler, JobParameter
 from .slurm import Slurm
 from .remote_slurm import RemoteSlurm
\ No newline at end of file
