@@ -1,6 +1,5 @@
 import itertools
 from enum import IntEnum
-from mstk import *
 from mstk.simsys import System
 from mstk.forcefield import *
 from mstk.topology import *
@@ -43,6 +42,7 @@ class OpenMMExporter:
         ----------
         system : System
         disable_inter_mol : bool
+            Disable inter-molecular interactions. This is useful for optimizing a batch of molecules
 
         Returns
         -------
@@ -51,18 +51,17 @@ class OpenMMExporter:
         if not OPENMM_FOUND:
             raise ImportError('Can not import OpenMM')
 
-        supported_terms = {LJ126Term, MieTerm,
-                           HarmonicBondTerm, MorseBondTerm,
-                           HarmonicAngleTerm, SDKAngleTerm, LinearAngleTerm,
+        supported_terms = {LJ126Term, MieTerm, MorseVdwTerm,
+                           HarmonicBondTerm, QuarticBondTerm, MorseBondTerm,
+                           HarmonicAngleTerm, QuarticAngleTerm, HarmonicCosineAngleTerm, SDKAngleTerm, LinearAngleTerm,
                            OplsDihedralTerm, PeriodicDihedralTerm,
                            OplsImproperTerm, HarmonicImproperTerm,
-                           DrudeTerm}
-        unsupported = system.ff_classes - supported_terms
+                           DrudePolarTerm}
+        unsupported = system.ff.energy_term_classes - supported_terms
         if unsupported != set():
-            raise Exception('Unsupported FF terms: %s'
-                            % (', '.join(map(lambda x: x.__name__, unsupported))))
+            raise Exception('Unsupported FF terms: %s' % (', '.join(map(lambda x: x.__name__, unsupported))))
 
-        if system.vsite_types - {TIP4PSite} != set():
+        if system.topology.virtual_site_classes - {TIP4PSite} != set():
             raise Exception('Virtual sites other than TIP4PSite haven\'t been implemented')
 
         omm_system = mm.System()
@@ -89,7 +88,7 @@ class OpenMMExporter:
         ff = system.ff
 
         ### Set up bonds #######################################################################
-        for bond_class in system.bond_classes:
+        for bond_class in system.ff.bond_term_classes:
             if bond_class == HarmonicBondTerm:
                 logger.debug('Setting up harmonic bonds...')
                 bforce = mm.HarmonicBondForce()
@@ -103,6 +102,23 @@ class OpenMMExporter:
                     if type(bterm) != HarmonicBondTerm:
                         continue
                     bforce.addBond(bond.atom1.id, bond.atom2.id, bterm.length, bterm.k * 2)
+            elif bond_class == QuarticBondTerm:
+                logger.debug('Setting up quartic bonds...')
+                bforce = mm.CustomBondForce('k2*(r-b0)^2+k3*(r-b0)^3+k4*(r-b0)^4')
+                bforce.addPerBondParameter('b0')
+                bforce.addPerBondParameter('k2')
+                bforce.addPerBondParameter('k3')
+                bforce.addPerBondParameter('k4')
+                for bond in top.bonds:
+                    if bond.is_drude:
+                        # DrudeForce will handle the bond between Drude pair
+                        continue
+                    if bond in system.constrain_bonds:
+                        continue
+                    bterm = system.bond_terms[bond]
+                    if type(bterm) != QuarticBondTerm:
+                        continue
+                    bforce.addBond(bond.atom1.id, bond.atom2.id, [bterm.length, bterm.k2, bterm.k3, bterm.k4])
             elif bond_class == MorseBondTerm:
                 logger.debug('Setting up Morse bonds...')
                 bforce = mm.CustomBondForce('depth*(1-exp(-alpha*(r-b0)))^2;'
@@ -121,7 +137,7 @@ class OpenMMExporter:
                         continue
                     bforce.addBond(bond.atom1.id, bond.atom2.id, [bterm.length, bterm.k, bterm.depth])
             else:
-                raise Exception('Bond terms other that HarmonicBondTerm and MorseBondTerm '
+                raise Exception('Bond terms other that HarmonicBondTerm, QuarticBondTerm and MorseBondTerm '
                                 'haven\'t been implemented')
             bforce.setUsesPeriodicBoundaryConditions(system.use_pbc)
             bforce.setForceGroup(ForceGroup.BOND)
@@ -129,7 +145,7 @@ class OpenMMExporter:
             omm_system.addForce(bforce)
 
         ### Set up angles #######################################################################
-        for angle_class in system.angle_classes:
+        for angle_class in system.ff.angle_term_classes:
             if angle_class == HarmonicAngleTerm:
                 logger.debug('Setting up harmonic angles...')
                 aforce = mm.HarmonicAngleForce()
@@ -138,8 +154,32 @@ class OpenMMExporter:
                         continue
                     aterm = system.angle_terms[angle]
                     if type(aterm) == HarmonicAngleTerm:
+                        aforce.addAngle(angle.atom1.id, angle.atom2.id, angle.atom3.id, aterm.theta, aterm.k * 2)
+            elif angle_class == QuarticAngleTerm:
+                logger.debug('Setting up quartic angles...')
+                aforce = mm.CustomAngleForce('k2*(theta-theta0)^2+k3*(theta-theta0)^3+k4*(theta-theta0)^4')
+                aforce.addPerAngleParameter('theta0')
+                aforce.addPerAngleParameter('k2')
+                aforce.addPerAngleParameter('k3')
+                aforce.addPerAngleParameter('k4')
+                for angle in top.angles:
+                    if angle in system.constrain_angles:
+                        continue
+                    aterm = system.angle_terms[angle]
+                    if type(aterm) == QuarticAngleTerm:
                         aforce.addAngle(angle.atom1.id, angle.atom2.id, angle.atom3.id,
-                                        aterm.theta, aterm.k * 2)
+                                        [aterm.theta, aterm.k2, aterm.k3, aterm.k4])
+            elif angle_class == HarmonicCosineAngleTerm:
+                logger.debug('Setting up harmonic cosine angles...')
+                aforce = mm.CustomAngleForce('k/sin(theta0)^2*(cos(theta)-cos(theta0))^2')
+                aforce.addPerAngleParameter('theta0')
+                aforce.addPerAngleParameter('k')
+                for angle in top.angles:
+                    if angle in system.constrain_angles:
+                        continue
+                    aterm = system.angle_terms[angle]
+                    if type(aterm) == HarmonicCosineAngleTerm:
+                        aforce.addAngle(angle.atom1.id, angle.atom2.id, angle.atom3.id, [aterm.theta, aterm.k])
             elif angle_class == SDKAngleTerm:
                 logger.debug('Setting up SDK angles...')
                 aforce = mm.CustomCompoundBondForce(
@@ -188,8 +228,8 @@ class OpenMMExporter:
                     a, k_linear = aterm.calc_a_k_linear(bterm12.length, bterm23.length)
                     aforce.addBond([angle.atom1.id, angle.atom2.id, angle.atom3.id], [a, k_linear])
             else:
-                raise Exception('Angle terms other that HarmonicAngleTerm, SDKAngleTerm and LinearAngleTerm '
-                                'haven\'t been implemented')
+                raise Exception('Angle terms other that HarmonicAngleTerm, QuarticAngleTerm, HarmonicCosineAngleTerm, '
+                                'SDKAngleTerm and LinearAngleTerm haven\'t been implemented')
             aforce.setUsesPeriodicBoundaryConditions(system.use_pbc)
             aforce.setForceGroup(ForceGroup.ANGLE)
             aforce.setName('Angle')
@@ -208,7 +248,7 @@ class OpenMMExporter:
                                          system.constrain_angles[angle])
 
         ### Set up dihedrals ###################################################################
-        for dihedral_class in system.dihedral_classes:
+        for dihedral_class in system.ff.dihedral_term_classes:
             if dihedral_class in (OplsDihedralTerm, PeriodicDihedralTerm):
                 logger.debug('Setting up periodic dihedrals...')
                 dforce = mm.PeriodicTorsionForce()
@@ -230,7 +270,7 @@ class OpenMMExporter:
             omm_system.addForce(dforce)
 
         ### Set up impropers ####################################################################
-        for improper_class in system.improper_classes:
+        for improper_class in system.ff.improper_term_classes:
             if improper_class == OplsImproperTerm:
                 logger.debug('Setting up periodic impropers...')
                 iforce = mm.CustomTorsionForce('k*(1-cos(2*theta))')
@@ -293,7 +333,7 @@ class OpenMMExporter:
         atom_types = list(ff.atom_types.values())
         type_names = list(ff.atom_types.keys())
         n_type = len(atom_types)
-        for vdw_class in system.vdw_classes:
+        for vdw_class in system.ff.vdw_term_classes:
             if vdw_class == LJ126Term:
                 logger.debug('Setting up LJ-12-6 vdW interactions...')
                 if system.use_pbc and ff.vdw_long_range == ForceField.VDW_LONGRANGE_SHIFT:
@@ -326,6 +366,41 @@ class OpenMMExporter:
                     id_type = type_names.index(atom.type)
                     cforce.addParticle([id_type])
 
+            elif vdw_class == MorseVdwTerm:
+                logger.debug('Setting up Morse vdW interactions...')
+                if system.use_pbc and ff.vdw_long_range == ForceField.VDW_LONGRANGE_SHIFT:
+                    cforce = mm.CustomNonbondedForce(
+                        f'DEPTH(type1,type2)*(1-exp(-ALPHA(type1,type2)*(r-R0(type1,type2))))^2-'
+                        f'DEPTH(type1,type2)*(1-exp(-ALPHA(type1,type2)*({cutoff}-R0(type1,type2))))^2'
+                    )
+                else:
+                    cforce = mm.CustomNonbondedForce(
+                        f'DEPTH(type1,type2)*((1-exp(-ALPHA(type1,type2)*(r-R0(type1,type2))))^2-1)'
+                    )
+                cforce.addPerParticleParameter('type')
+                DEPTH_list = [0.0] * n_type * n_type
+                ALPHA_list = [0.0] * n_type * n_type
+                R0_list = [0.0] * n_type * n_type
+                for i, atype1 in enumerate(atom_types):
+                    for j, atype2 in enumerate(atom_types):
+                        vdw = ff.get_vdw_term(atype1, atype2)
+                        if type(vdw) == MorseVdwTerm:
+                            DEPTH = vdw.depth
+                            ALPHA = vdw.alpha
+                            R0 = vdw.r0
+                        else:
+                            DEPTH = ALPHA = R0 = 0
+                        DEPTH_list[i + n_type * j] = DEPTH
+                        ALPHA_list[i + n_type * j] = ALPHA
+                        R0_list[i + n_type * j] = R0
+                cforce.addTabulatedFunction('DEPTH', mm.Discrete2DFunction(n_type, n_type, DEPTH_list))
+                cforce.addTabulatedFunction('ALPHA', mm.Discrete2DFunction(n_type, n_type, ALPHA_list))
+                cforce.addTabulatedFunction('R0', mm.Discrete2DFunction(n_type, n_type, R0_list))
+
+                for atom in top.atoms:
+                    id_type = type_names.index(atom.type)
+                    cforce.addParticle([id_type])
+
             elif vdw_class == MieTerm:
                 logger.debug('Setting up Mie vdW interactions...')
                 if system.use_pbc and ff.vdw_long_range == ForceField.VDW_LONGRANGE_SHIFT:
@@ -369,7 +444,7 @@ class OpenMMExporter:
                     cforce.addParticle([id_type])
 
             else:
-                raise Exception('vdW terms other than LJ126Term and MieTerm '
+                raise Exception('vdW terms other than LJ126Term, MieTerm and MorseVdwTerm '
                                 'haven\'t been implemented')
             if system.use_pbc:
                 cforce.setNonbondedMethod(mm.CustomNonbondedForce.CutoffPeriodic)
@@ -425,13 +500,27 @@ class OpenMMExporter:
                     elif type(vdw) == MieTerm:
                         cbforce.addBond(atom1.id, atom2.id,
                                         [epsilon, vdw.sigma, vdw.repulsion, vdw.attraction])
+                elif type(vdw) is MorseVdwTerm:
+                    cbforce = pair14_forces.get(MorseVdwTerm)
+                    if cbforce is None:
+                        cbforce = mm.CustomBondForce('DEPTH*((1-exp(-ALPHA*(r-R0)))^2-1)')
+                        cbforce.addPerBondParameter('DEPTH')
+                        cbforce.addPerBondParameter('ALPHA')
+                        cbforce.addPerBondParameter('R0')
+                        cbforce.setUsesPeriodicBoundaryConditions(system.use_pbc)
+                        cbforce.setForceGroup(ForceGroup.VDW)
+                        cbforce.setName('vdW')
+                        omm_system.addForce(cbforce)
+                        pair14_forces[MorseVdwTerm] = cbforce
+                    depth = vdw.depth * ff.scale_14_vdw
+                    cbforce.addBond(atom1.id, atom2.id, [depth, vdw.alpha, vdw.r0])
                 else:
-                    raise Exception('1-4 scaling for vdW terms other than LJ126Term and MieTerm '
+                    raise Exception('1-4 scaling for vdW terms other than LJ126Term, MieTerm and MorseVdwTerm '
                                     'haven\'t been implemented')
 
         ### Set up Drude particles ##############################################################
-        for polar_class in system.polarizable_classes:
-            if polar_class == DrudeTerm:
+        for polar_class in system.ff.polar_term_classes:
+            if polar_class == DrudePolarTerm:
                 logger.debug('Setting up Drude polarizations...')
                 pforce = mm.DrudeForce()
                 pforce.setForceGroup(ForceGroup.DRUDE)
@@ -439,7 +528,7 @@ class OpenMMExporter:
                 omm_system.addForce(pforce)
                 parent_idx_thole = {}  # {parent: (index in DrudeForce, thole)} for addScreenPair
                 for parent, drude in system.drude_pairs.items():
-                    pterm = system.polarizable_terms[parent]
+                    pterm = system.polar_terms[parent]
                     n_H = len([atom for atom in parent.bond_partners if atom.symbol == 'H'])
                     alpha = pterm.alpha + n_H * pterm.merge_alpha_H
                     idx = pforce.addParticle(drude.id, parent.id, -1, -1, -1,
@@ -486,7 +575,7 @@ class OpenMMExporter:
                     for f in custom_nb_forces:
                         f.addExclusion(a1.id, a2.id)
             else:
-                raise Exception('Polarizable terms other that DrudeTerm haven\'t been implemented')
+                raise Exception('Polar terms other that DrudePolarTerm haven\'t been implemented')
 
         ### Set up virtual sites ################################################################
         if top.has_virtual_site:
@@ -555,10 +644,10 @@ class OpenMMExporter:
 
     @staticmethod
     def _setup_nonbonded_as_bonded(system, omm_system):
-        if system.polarizable_classes != set():
-            raise Exception('Polarizable terms not supported')
+        if system.ff.polar_term_classes:
+            raise Exception('Polar terms not supported')
 
-        if system.vsite_types != set():
+        if system.topology.virtual_site_classes:
             raise Exception('Virtual sites not supported')
 
         if system.use_pbc:
@@ -589,6 +678,7 @@ class OpenMMExporter:
 
         for mol in top.molecules:
             pairs12, pairs13, pairs14 = mol.get_12_13_14_pairs()
+            # each pair tuple (Atom, Atom) in each pairs list are sorted by the Atom.id attributes of two atoms
             pairs1n = list(set(itertools.combinations(mol.atoms, 2)) - set(pairs12).union(pairs13).union(pairs14))
             for i, (atom1, atom2) in enumerate(pairs14 + pairs1n):
                 vdw = ff.get_vdw_term(ff.atom_types[atom1.type], ff.atom_types[atom2.type])
@@ -601,4 +691,6 @@ class OpenMMExporter:
                     ljforce.addBond(atom1.id, atom2.id, [epsilon, vdw.sigma, 12, 6])
                 elif type(vdw) == MieTerm:
                     ljforce.addBond(atom1.id, atom2.id, [epsilon, vdw.sigma, vdw.repulsion, vdw.attraction])
+                else:
+                    raise Exception(f'Cannot setup {vdw} as a bonded term')
                 qqforce.addBond(atom1.id, atom2.id, [qq])
