@@ -1,4 +1,3 @@
-import copy
 import itertools
 from ..topology import *
 from ..forcefield import *
@@ -9,9 +8,10 @@ class System:
     '''
     System is a combination of topology and forcefield, ready for simulation.
 
-    When a system is being initiated, the topology is deep copied.
-    A compressed set of force field is generated, which only contains the terms required by the topology.
-    Any modification to the topology and force field after that will have no effect on the system.
+    When a system being initialized, a compressed set of force field is constructed with the terms required by the topology.
+    Note that the terms in the compressed force field are references to the terms in the original force field.
+    A series of data structures are also constructed to map the topological elements to force field terms.
+    Unless you are certain, DO NOT make any modification to the topology and force field after a system is initialized, as it may corrupt the system.
 
     During the initialization, all information in the topology will be kept untouched, which means:
 
@@ -24,34 +24,27 @@ class System:
     * If this is a Drude polarizable model, the Drude particles must be available in the topology,
       and the polarizability, thole screening and charges on Drude particles must have been correctly assigned.
       You may need to call :func:`mstk.topology.Topology.generate_drude_particles` to generate Drude particles.
-    * If this is a virtual site model, the virtual site particles must be available in the topology,
+    * If this is a virtual site model, the virtual site particles must be available in the topology.
 
     The provided force field must be able to fully describe the energy of the topology.
-    If any term required by topology is not found in the force field, an Exception will be raised.
+    If any atom type from topology is not found in the FF, an Exception will be raised.
+    If any vdw/bond/angle/dihedral/improper/polar term required by topology is not found in the FF,
+    an Exception will be raised unless `allow_missing_terms` set to True.
+    It won't check charge increment terms and virtual site terms, because they are already represented by the topology itself.
 
-    Positions and unit cell are usually included in the topology.
-    If not so, they can be provided by arguments `positions` and `cell`.
-    The explicitly provided positions and cell will override the ones in topology.
-    If positions are not included in the topology and not provided explicitly, an Exception will be raised.
+    Positions and unit cell should be included in the topology.
+    If positions are not included in the topology, an Exception will be raised.
     Unit cell is optional. If unit cell is not provided, cutoff will not be used for non-bonded interactions.
 
-    If some bonded (bond, angle, dihedral, improper) parameters required by the topology are not found in the force field,
-    `mstk` can try to transfer parameters from more general terms in the force field.
-    e.g. if bond term between 'c_4o2' and 'n_3' is not found, the bond term between 'c_4' and 'n_3' will be used.
-    This mechanism is initially designed for TEAM force field.
-    It is disabled by default, and can be enabled by setting argument `transfer_bonded_terms` to True.
-
     Parameters
     ----------
     topology : Topology
     ff : ForceField
-    positions : array_like, optional
-    cell : UnitCell, optional
-    ignore_missing_improper: bool
-        If set to True, a zero-energy OplsImproperTerm will be created for missing improper terms in the FF
-    transfer_bonded_terms: bool
-    supress_pbc_warning: bool
+    suppress_pbc_warning: bool
         Set to True if you intend to build a vacuum system and don't want to hear warning about it.
+    allow_missing_terms: bool
+        If set to True, won't raise Exception if FF terms other than AtomType are missing.
+        This is useful if you want to check which FF terms are missing.
 
     Attributes
     ----------
@@ -63,35 +56,21 @@ class System:
     vsite_pairs : dict, [Atom, Atom]
     constrain_bonds : dict, [Bond, float]
     constrain_angles : dict, [Angle, float]
+    atom_vdw_terms : dict, [Atom, subclass of VdwTerm]
     bond_terms : dict, [Bond, subclass of BondTerm]
     angle_terms : dict, [Angle, subclass of AngleTerm]
     dihedral_terms : dict, [Dihedral, subclass of DihedralTerm]
     improper_terms : dict, [Improper, subclass of ImproperTerm]
-    polarizable_terms :  dict, [Atom, subclass of PolarizableTerm]
-    vdw_classes : set of subclass of VdwTerm
-    bond_classes : set of subclass of BondTerm
-    angle_classes : set of subclass of AngleTerm
-    dihedral_classes : set of subclass of DihedralTerm
-    improper_classes : set of subclass of ImproperTerm
-    polarizable_classes : set of subclass of PolarizableTerm
-    ff_classes : set of subclass of FFTerm
-    vsite_types : set of subclass of VirtualSite
+    polar_terms :  dict, [Atom, subclass of PolarTerm]
+    missing_terms : list of FFTerm
     '''
 
-    def __init__(self, topology, ff, positions=None, cell=None,
-                 ignore_missing_improper=False,
-                 transfer_bonded_terms=False,
-                 suppress_pbc_warning=False):
-        self._topology = copy.deepcopy(topology)
+    def __init__(self, topology, ff, suppress_pbc_warning=False, allow_missing_terms=False):
+        self._topology = topology
         self._ff = ForceField()
 
-        if positions is not None:
-            self._topology.set_positions(positions)
-        elif not topology.has_position:
-            raise Exception('Positions should be provided with topology or positions')
-
-        if cell is not None:
-            self._topology.cell = UnitCell(cell.vectors)
+        if not topology.has_position:
+            raise Exception('Positions should be provided with topology')
 
         if self._topology.n_atom == 0:
             logger.error('No atoms in the topology')
@@ -119,27 +98,21 @@ class System:
         # prebuild topology information
         self.drude_pairs: {Atom: Atom} = dict(self._topology.get_drude_pairs())  # {parent: atom_drude}
         self.vsite_pairs: {Atom: Atom} = dict(self._topology.get_virtual_site_pairs())  # {parent: atom_vsite}
-        # records the types of VirtualSite in topology. It is NOT the VirtualSiteTerm in force field
-        self.vsite_types = set([type(atom.virtual_site) for atom in self.vsite_pairs.values()])
 
-        # bind FF terms with topological elements
+        # map topological elements to FF terms
+        self.atom_vdw_terms: {Atom: VdwTerm} = {}  # only same-type vdw terms are stored
         self.bond_terms: {Bond: BondTerm} = {}  # Drude bonds are not included
         self.angle_terms: {Angle: AngleTerm} = {}
         self.dihedral_terms: {Dihedral: DihedralTerm} = {}
         self.improper_terms: {Improper: ImproperTerm} = {}
-        self.polarizable_terms: {Atom: PolarizableTerm} = {}  # key is parent Atom
+        self.polar_terms: {Atom: PolarTerm} = {}  # key is parent Atom
         self.constrain_bonds: {Bond: float} = {}  # value is distance
         self.constrain_angles: {Angle: float} = {}  # value is 1-3 distance
 
-        self.vdw_classes = set()
-        self.bond_classes = set()
-        self.angle_classes = set()
-        self.dihedral_classes = set()
-        self.improper_classes = set()
-        self.polarizable_classes = set()
-        self.ff_classes = set()
+        # missing FF terms
+        self.missing_terms = []
 
-        self._extract_terms(ff, ignore_missing_improper, transfer_bonded_terms)
+        self._extract_terms(ff, allow_missing_terms)
 
     @property
     def topology(self):
@@ -163,20 +136,15 @@ class System:
         '''
         return self._ff
 
-    def _extract_terms(self, ff, ignore_missing_improper=False, transfer_bonded_terms=False):
+    def _extract_terms(self, ff, allow_missing_terms=False):
         '''
-        Extract only the required terms from a force field.
-
-        If any term required by topology is not found, an Exception will be raised.
+        Extract only the required terms from a force field, and record the missing terms.
 
         Parameters
         ----------
         ff : ForceField
-        ignore_missing_improper : bool
-        transfer_bonded_terms : bool
+        allow_missing_terms : bool
         '''
-        from mstk.forcefield.dff_utils import dff_fuzzy_match
-
         self._ff.restore_settings(ff.get_settings())
 
         _atype_not_found = set()
@@ -186,19 +154,18 @@ class System:
             except:
                 _atype_not_found.add(atom.type)
             else:
-                if atype.name not in self._ff.atom_types:
-                    self._ff.add_term(atype)
-        if _atype_not_found != set():
-            logger.error('%i atom types not found in FF: %s' % (
-                len(_atype_not_found), ' '.join(_atype_not_found)))
-            raise Exception('Parameters missing')
+                self._ff.add_term(atype, replace=True)
+        if _atype_not_found:
+            logger.error('%i atom types not found in FF: %s' % (len(_atype_not_found), ' '.join(_atype_not_found)))
+            raise Exception('Incomplete force field')
 
-        _vdw_not_found = set()
+        _vdw_not_found = {}
         for atype in self._ff.atom_types.values():
             try:
                 vdw = ff.get_vdw_term(atype, atype)
             except Exception as e:
-                _vdw_not_found.add(atype.eqt_vdw)
+                _vdw = VdwTerm(atype.eqt_vdw, atype.eqt_vdw)
+                _vdw_not_found[_vdw.name] = _vdw
             else:
                 self._ff.add_term(vdw, replace=True)
 
@@ -206,41 +173,44 @@ class System:
             try:
                 vdw = ff.get_vdw_term(atype1, atype2, mixing=False)
             except:
-                pass
+                if self._ff.lj_mixing_rule == self._ff.LJ_MIXING_NONE:
+                    # all pairwise vdW terms are explicitly required
+                    _vdw = VdwTerm(atype1.eqt_vdw, atype2.eqt_vdw)
+                    _vdw_not_found[_vdw.name] = _vdw
             else:
                 self._ff.add_term(vdw, replace=True)
 
-        _bterm_not_found = set()
-        _bterm_transferred = set()
+        # map atom to self vdW term
+        for atom in self._topology.atoms:
+            atype = self._ff.atom_types[atom.type]
+            try:
+                vdw = self._ff.get_vdw_term(atype, atype)
+            except FFTermNotFoundError:
+                pass
+            else:
+                self.atom_vdw_terms[atom] = vdw
+
+        _bterm_not_found = {}
         for bond in self._topology.bonds:
             if bond.is_drude:
                 # the Drude-parent bonds will be handled by DrudeForce below
                 continue
-            _found = False
             ats = ff.get_eqt_for_bond(bond)
             _term = BondTerm(*ats, 0)
             try:
                 bterm = ff.get_bond_term(*ats)
-                _found = True
-            except:
-                if transfer_bonded_terms:
-                    bterm, score = dff_fuzzy_match(_term, ff)
-                    if bterm is not None:
-                        _found = True
-                        _bterm_transferred.add((_term.name, bterm.name, score))
-            if not _found:
-                _bterm_not_found.add(_term.name)
+            except FFTermNotFoundError:
+                _bterm_not_found[_term.name] = _term
             else:
                 self._ff.add_term(bterm, replace=True)
                 self.bond_terms[bond] = bterm
                 if bterm.fixed:
                     self.constrain_bonds[bond] = bterm.length
 
-        _aterm_not_found = set()
-        _aterm_transferred = set()
+        _aterm_not_found = {}
         for angle in self._topology.angles:
             _found = False
-            ats_list = ff.get_eqt_for_angle(angle)
+            ats_list = ff.get_eqt_list_for_angle(angle)
             _term = AngleTerm(*ats_list[0], 0)
             for ats in ats_list:
                 try:
@@ -250,14 +220,8 @@ class System:
                 else:
                     _found = True
                     break
-            else:
-                if transfer_bonded_terms:
-                    aterm, score = dff_fuzzy_match(_term, ff)
-                    if aterm is not None:
-                        _found = True
-                        _aterm_transferred.add((_term.name, aterm.name, score))
             if not _found:
-                _aterm_not_found.add(_term.name)
+                _aterm_not_found[_term.name] = _term
             else:
                 self._ff.add_term(aterm, replace=True)
                 self.angle_terms[angle] = aterm
@@ -272,12 +236,11 @@ class System:
                     d1, d2 = self.constrain_bonds[bond1], self.constrain_bonds[bond2]
                     self.constrain_angles[angle] = math.sqrt(d1 * d1 + d2 * d2 - 2 * d1 * d2 * math.cos(aterm.theta))
 
-        _dterm_not_found = set()
-        _dterm_transferred = set()
+        _dterm_not_found = {}
         for mol in self._topology.molecules:
             for dihedral in mol.dihedrals:
                 _found = False
-                ats_list = ff.get_eqt_for_dihedral(dihedral)
+                ats_list = ff.get_eqt_list_for_dihedral(dihedral)
                 _term = DihedralTerm(*ats_list[0])
                 for ats in ats_list:
                     try:
@@ -287,23 +250,16 @@ class System:
                     else:
                         _found = True
                         break
-                else:
-                    if transfer_bonded_terms:
-                        dterm, score = dff_fuzzy_match(_term, ff)
-                        if dterm is not None:
-                            _found = True
-                            _dterm_transferred.add((_term.name, dterm.name, score))
                 if not _found:
-                    _dterm_not_found.add(_term.name)
+                    _dterm_not_found[_term.name] = _term
                 else:
                     self._ff.add_term(dterm, replace=True)
                     self.dihedral_terms[dihedral] = dterm
 
-        _iterm_not_found = set()
-        _iterm_transferred = set()
+        _iterm_not_found = {}
         for improper in self._topology.impropers:
             _found = False
-            ats_list = ff.get_eqt_for_improper(improper)
+            ats_list = ff.get_eqt_list_for_improper(improper)
             _term = ImproperTerm(*ats_list[0])
             for ats in ats_list:
                 try:
@@ -313,76 +269,36 @@ class System:
                 else:
                     _found = True
                     break
-            else:
-                if transfer_bonded_terms:
-                    iterm, score = dff_fuzzy_match(_term, ff)
-                    if iterm is not None:
-                        _found = True
-                        _iterm_transferred.add((_term.name, iterm.name, score))
             if not _found:
-                if ignore_missing_improper:
-                    iterm = OplsImproperTerm(*ats_list[0], 0.0)
-                    _found = True
-                else:
-                    _iterm_not_found.add(_term.name)
-            if _found:
+                _iterm_not_found[_term.name] = _term
+            else:
                 self._ff.add_term(iterm, replace=True)
                 self.improper_terms[improper] = iterm
 
-        _pterm_not_found = set()
+        _pterm_not_found = {}
         for parent in self.drude_pairs.keys():
-            _term = PolarizableTerm(ff.atom_types[parent.type].eqt_polar)
+            _term = PolarTerm(ff.atom_types[parent.type].eqt_polar)
             try:
-                pterm = ff.polarizable_terms[_term.name]
+                pterm = ff.polar_terms[_term.name]
             except:
-                _pterm_not_found.add(_term.name)
+                _pterm_not_found[_term.name] = _term
             else:
                 self._ff.add_term(pterm, replace=True)
-                self.polarizable_terms[parent] = pterm
-
-        ### print all transferred terms
-        _n_transferred = 0
-        msg = ''
-        for k, v in {'Bond'    : _bterm_transferred,
-                     'Angle'   : _aterm_transferred,
-                     'Dihedral': _dterm_transferred,
-                     'Improper': _iterm_transferred}.items():
-            for i in v:
-                _n_transferred += 1
-                msg += '        %-10s %-20s %-20s %i\n' % (k, *i)
-        if _n_transferred > 0:
-            logger.warning(
-                '%i bonded terms not found in FF. Transferred from similar terms with score\n' % _n_transferred + msg)
+                self.polar_terms[parent] = pterm
 
         ### print all the missing terms
-        _n_missing = 0
-        msg = ''
-        for k, v in {'vdW'        : _vdw_not_found,
-                     'Bond'       : _bterm_not_found,
-                     'Angle'      : _aterm_not_found,
-                     'Dihedral'   : _dterm_not_found,
-                     'Improper'   : _iterm_not_found,
-                     'Polarizable': _pterm_not_found}.items():
-            for i in v:
-                _n_missing += 1
-                msg += '        %-10s %-20s\n' % (k, i)
-        if _n_missing > 0:
-            logger.error('%i terms not found in FF\n' % _n_missing + msg)
-            raise Exception('Parameter missing')
+        for d in _vdw_not_found, _bterm_not_found, _aterm_not_found, _dterm_not_found, _iterm_not_found, _pterm_not_found:
+            self.missing_terms.extend(d.values())
 
-        self.vdw_classes = {term.__class__ for term in self._ff.vdw_terms.values()}
-        self.vdw_classes.update({term.__class__ for term in self._ff.pairwise_vdw_terms.values()})
-        self.bond_classes = {term.__class__ for term in self.bond_terms.values()}
-        self.angle_classes = {term.__class__ for term in self.angle_terms.values()}
-        self.dihedral_classes = {term.__class__ for term in self.dihedral_terms.values()}
-        self.improper_classes = {term.__class__ for term in self.improper_terms.values()}
-        self.polarizable_classes = {term.__class__ for term in self.polarizable_terms.values()}
-        self.ff_classes = (self.vdw_classes
-                           .union(self.bond_classes)
-                           .union(self.angle_classes)
-                           .union(self.dihedral_classes)
-                           .union(self.improper_classes)
-                           .union(self.polarizable_classes))
+        if self.missing_terms:
+            msg = f'{len(self.missing_terms)} terms not found in FF'
+            for term in self.missing_terms:
+                msg += f'\n        {term}'
+            if allow_missing_terms:
+                logger.warning(msg)
+            else:
+                logger.error(msg)
+                raise Exception('Incomplete force field')
 
     def export_lammps(self, data_out='data.lmp', in_out='in.lmp', **kwargs):
         '''
@@ -429,6 +345,7 @@ class System:
         Parameters
         ----------
         disable_inter_mol : bool
+            Disable inter-molecular interactions. This is useful for optimizing a batch of molecules
 
         Returns
         -------
@@ -437,10 +354,15 @@ class System:
         from .ommexporter import OpenMMExporter
         return OpenMMExporter.export(self, disable_inter_mol=disable_inter_mol, **kwargs)
 
-    def decompose_energy(self, verbose=True):
+    def decompose_energy(self, disable_inter_mol=False, verbose=True):
         '''
         Calculate the contribution of each energy term in this system
 
+        Parameters
+        ----------
+        disable_inter_mol : bool
+        verbose : bool
+
         Returns
         -------
         energies : Dict[str, float]
@@ -452,18 +374,24 @@ class System:
         except ImportError:
             raise Exception('OpenMM is required for energy decomposition')
 
-        omm_sys = self.to_omm_system()
+        omm_sys = self.to_omm_system(disable_inter_mol=disable_inter_mol)
         integrator = mm.VerletIntegrator(0.001)
         platform = mm.Platform.getPlatformByName('CPU')
         sim = app.Simulation(self.topology.to_omm_topology(), omm_sys, integrator, platform)
         sim.context.setPositions(self.topology.positions)
         return energy_decomposition(sim, logger=logger if verbose else None)
 
-    def minimize_energy(self, tolerance=100, verbose=True):
+    def minimize_energy(self, disable_inter_mol=False, tolerance=100.0, verbose=True):
         '''
-        Perform energy minimization on this system
+        Perform energy minimization on this system.
 
-        The position of each atom in the topology will be updated
+        The position of each atom in the topology will be updated.
+
+        Parameters
+        ----------
+        disable_inter_mol : bool
+        tolerance : float
+        verbose : bool
         '''
         try:
             from openmm import openmm as mm, app
@@ -471,7 +399,7 @@ class System:
         except ImportError:
             raise Exception('OpenMM is required for energy minimization')
 
-        omm_sys = self.to_omm_system()
+        omm_sys = self.to_omm_system(disable_inter_mol=disable_inter_mol)
         integrator = mm.VerletIntegrator(0.001)
         platform = mm.Platform.getPlatformByName('CPU')
         sim = app.Simulation(self.topology.to_omm_topology(), omm_sys, integrator, platform)
