@@ -129,7 +129,7 @@ class Bond():
         value : float
         '''
         if cell:
-            return periodic_distance(self.atom1.position, self.atom2.position, cell.size)
+            return periodic_distance(self.atom1.position, self.atom2.position, cell.get_size())
 
         delta = self.atom2.position - self.atom1.position
         return float(np.sqrt(delta.dot(delta)))
@@ -264,7 +264,7 @@ class Angle():
         value : float
         '''
         if cell:
-            return periodic_angle(self.atom1.position, self.atom2.position, self.atom3.position, cell)
+            return periodic_angle(self.atom1.position, self.atom2.position, self.atom3.position, cell.get_size())
 
         vec1 = self.atom1.position - self.atom2.position
         vec2 = self.atom3.position - self.atom2.position
@@ -289,6 +289,9 @@ class Dihedral():
     atom2 : Atom
     atom3 : Atom
     atom4 : Atom
+    override_atom_types: list of str
+        When matching dihedral terms from FF, the type attributes of the four atoms will be overridden with these specified types.
+        Each element can be a string or None. If None, the type attribute of the corresponding atom will be used, and not get overridden.
     '''
 
     def __init__(self, atom1, atom2, atom3, atom4):
@@ -296,6 +299,7 @@ class Dihedral():
         self.atom2 = atom2
         self.atom3 = atom3
         self.atom4 = atom4
+        self.override_atom_types = [None, None, None, None]
 
     def __repr__(self):
         return '<Dihedral: %s>' % self.name
@@ -408,7 +412,7 @@ class Dihedral():
         '''
         if cell:
             return periodic_dihedral(self.atom1.position, self.atom2.position, self.atom3.position, self.atom4.position,
-                                     cell.size)
+                                     cell.get_size())
 
         vec1 = self.atom2.position - self.atom1.position
         vec2 = self.atom3.position - self.atom2.position
@@ -506,7 +510,7 @@ class Improper():
         '''
         if cell:
             return periodic_dihedral(self.atom1.position, self.atom2.position, self.atom3.position, self.atom4.position,
-                                     cell.size)
+                                     cell.get_size())
 
         vec1 = self.atom2.position - self.atom1.position
         vec2 = self.atom3.position - self.atom2.position
