@@ -1,4 +1,5 @@
 import os
+import shutil
 import numpy as np
 import copy
 import tempfile
@@ -160,21 +161,22 @@ class Topology():
         '''
         Enlarge the number of molecules in this topology with Packmol.
 
-        It is used for building real simulation box from several different molecules.
-        It generates molecules with appropriate positions and addes them into the topology.
+        It is used for building real simulation box from several molecules.
+        It generates molecules with appropriate positions and adds them into the topology.
         The unit cell should already be defined, and it will not be touched during the scaling.
 
+        An input file for packmol and xyz files for molecules will be created under `tempdir`.
         If packmol is provided, then it will be invoked to pack the simulation box,
-        and the generated positions will be loaded back to the topology.
+        and the generated positions will be loaded back to the topology,
+        and the `tempdir` will be removed.
         Otherwise, the topology will be enlarged without correct positions,
-        and a input file for packmol and xyz files for molecules will be saved in current folder
-        so that positions can be built manually and loaded back to the topology later
+        and the files under `tempdir` will be kept so that positions can be built manually and loaded back to the topology later.
 
         Parameters
         ----------
         numbers : int or list of int
             If numbers is a int, the topology will be scaled by this times.
-            If numbers is a list of int, the each molecule will be scaled by corresponding times.
+            If numbers is a list of int, then each molecule will be scaled by corresponding times.
         packmol : Packmol, Optional
             If is None, will generate input files for packmol.
         seed : int
@@ -214,9 +216,9 @@ class Topology():
 
         tmp_inp = tempdir + '/_pack.inp'
         tmp_out = tempdir + '/_out.xyz'
+        box = self.cell.get_size()
         if packmol is not None:
-            packmol.build_box(xyz_files, numbers, tmp_out,
-                              size=self.cell.size - 0.2, inp_file=tmp_inp, seed=seed, silent=True)
+            packmol.build_box(xyz_files, numbers, tmp_out, size=box - 0.2, inp_file=tmp_inp, seed=seed, silent=True)
             try:
                 xyz = Topology.open(tmp_out)
             except:
@@ -227,8 +229,9 @@ class Topology():
 
             self.update_molecules(self._molecules, numbers)
             self.set_positions(xyz.positions)
+            shutil.rmtree(tempdir)
         else:
-            Packmol.gen_inp(xyz_files, numbers, tmp_out, size=self.cell.size - 0.2, inp_file=tmp_inp, seed=seed)
+            Packmol.gen_inp(xyz_files, numbers, tmp_out, size=box - 0.2, inp_file=tmp_inp, seed=seed)
 
     def scale_box(self, scale, rigid_group='atom'):
         '''
@@ -465,6 +468,17 @@ class Topology():
         '''
         return any(atom.virtual_site is not None for atom in self._atoms)
 
+    @property
+    def virtual_site_classes(self):
+        '''
+        A set of all the VirtualSite classes in the topology
+
+        Returns
+        -------
+        classes : set of subclass of VirtualSite
+        '''
+        return set(type(atom.virtual_site) for atom in self._atoms if atom.virtual_site is not None)
+
     @staticmethod
     def open(file, **kwargs):
         '''
@@ -678,7 +692,7 @@ class Topology():
 
     def generate_drude_particles(self, ff, **kwargs):
         '''
-        Generate Drude particles from DrudeTerms in a polarizable force field
+        Generate Drude particles from DrudePolarTerms in a polarizable force field
 
         Parameters
         ----------
